// importer la bibliothèque video de processing
import processing.video.*;
// importer la bibliothèque blobDetection
import blobDetection.*;

// je créé un objet de capture appelé 'cam'
Capture cam;
//je créée un objet de detection des blobs appelé 'theBlobDetection'
BlobDetection theBlobDetection;
// on créée une variable image appelée 'img'
//
PImage img;
//on créé une variable booléenne pour la détection des blobs
boolean newFrame=false;
//notre variable qui nous permettra de regler le contraste
// de captation en deplacant notre curseur de souris
float contraste;


void setup()
{
  // taille du sketch
  size(640, 480);
  // Capture
  cam = new Capture(this, 40*4, 30*4, 15);
  // BlobDetection
  // img est l'image qui va servir à détecter nos blobs
  //on enverra cette image comme copie de la capture de la webcam afin
  // d'assurer un traitement plus rapide
  img = new PImage(80, 60); 
  // on assigne les parametres à notre objet 'theBlobDetection'
  // on lui dit de detecter les blobs sur l'ensemble de l'image
  theBlobDetection = new BlobDetection(img.width, img.height);
  theBlobDetection.setPosDiscrimination(true);
 
}

void draw()
{
   // on regle le seuil de luminosité pour la detection des blobs
  theBlobDetection.setThreshold(contraste); //O.2f
  //a chaque nouvelle image captée par la webcam
  if (newFrame)
  {
    newFrame=false;
    // on récupere le cadre de la la webcam
    image(cam, 0, 0, width, height);
    // on la copie pour la traiter
    img.copy(cam, 0, 0, cam.width, cam.height, 
    0, 0, img.width, img.height);
    //on active la fonction fastblur
    fastblur(img, 2);
    //on lance l'analyse des blobs
    theBlobDetection.computeBlobs(img.pixels);
    //on demande de dessiner les lignes vertes et les rectangles rouges
    drawBlobsAndEdges(true, true);
  }
  println(contraste);
}

// ==================================================
// drawBlobsAndEdges()
// ==================================================
// fonction de detection des contours des blobs et leur dessin
// (lignes vertes)
void drawBlobsAndEdges(boolean drawBlobs, boolean drawEdges)
{
  //on desactive le remplissage de couleur
  //pour les formes
  //afin de ne voir que les contours
  noFill();
  //
  Blob b;
  EdgeVertex eA, eB;
  for (int n=0 ; n<theBlobDetection.getBlobNb() ; n++)
  {
    b=theBlobDetection.getBlob(n);
    //s'il des blobs existent dans notre cadre
    if (b!=null)
    {
      // Edges
      if (drawEdges)
      {
        //epaisseur des contours
        strokeWeight(3);
        // couleur verte pour les contours
        stroke(0, 255, 0);
        
        for (int m=0;m<b.getEdgeNb();m++)
        {
          eA = b.getEdgeVertexA(m);
          eB = b.getEdgeVertexB(m);
          if (eA !=null && eB !=null)
            line(
            eA.x*width, eA.y*height, 
            eB.x*width, eB.y*height
              );
        }
      }

      // Blobs
      if (drawBlobs)
      {
        //epaisseur des contours
        strokeWeight(1);
        // couleur rouge pour les contours des rectangles
        stroke(255, 0, 0);
        //on dessine les rectangles en prenant 
//        rect(
//        b.xMin*width, b.yMin*height, 
//        b.w*width, b.h*height
//          );
// ondesssine nos cercles rouges
// on recupere leur coordonnées des centres de gravité des blobs avec :
// b.xMin*width+b.w*width/2 pour les X
// b.yMin*height+b.h*height/2 pour les Y
// le diametre est de 10
fill(255,0,0);
ellipse(b.xMin*width+b.w*width/2, b.yMin*height+b.h*height/2, 10, 10);
      }
    }
  }
}

//notre méthode qui permet de régler
// le contraste de capture
//en deplacant notre curseur de souris
//horizontalement
void mouseDragged() {
  // quand on déplace la souris, on change la valeur du contraste 
  contraste =  map(mouseX, 0, width, 0, 0.6);
}


// méthode de capture de la webcam
// nous n'interviendrons pas sur cette fonction
void captureEvent(Capture cam)
{
  cam.read();
  newFrame = true;
}

// ==================================================
// Super Fast Blur v1.1
// by Mario Klingemann 
// <http://incubator.quasimondo.com>
// ==================================================
// une fonction complémentaire du traitement de l'image sur lequel nous n'interviendrons pas
void fastblur(PImage img, int radius)
{
  if (radius<1) {
    return;
  }
  int w=img.width;
  int h=img.height;
  int wm=w-1;
  int hm=h-1;
  int wh=w*h;
  int div=radius+radius+1;
  int r[]=new int[wh];
  int g[]=new int[wh];
  int b[]=new int[wh];
  int rsum, gsum, bsum, x, y, i, p, p1, p2, yp, yi, yw;
  int vmin[] = new int[max(w, h)];
  int vmax[] = new int[max(w, h)];
  int[] pix=img.pixels;
  int dv[]=new int[256*div];
  for (i=0;i<256*div;i++) {
    dv[i]=(i/div);
  }

  yw=yi=0;

  for (y=0;y<h;y++) {
    rsum=gsum=bsum=0;
    for (i=-radius;i<=radius;i++) {
      p=pix[yi+min(wm, max(i, 0))];
      rsum+=(p & 0xff0000)>>16;
      gsum+=(p & 0x00ff00)>>8;
      bsum+= p & 0x0000ff;
    }
    for (x=0;x<w;x++) {

      r[yi]=dv[rsum];
      g[yi]=dv[gsum];
      b[yi]=dv[bsum];

      if (y==0) {
        vmin[x]=min(x+radius+1, wm);
        vmax[x]=max(x-radius, 0);
      }
      p1=pix[yw+vmin[x]];
      p2=pix[yw+vmax[x]];

      rsum+=((p1 & 0xff0000)-(p2 & 0xff0000))>>16;
      gsum+=((p1 & 0x00ff00)-(p2 & 0x00ff00))>>8;
      bsum+= (p1 & 0x0000ff)-(p2 & 0x0000ff);
      yi++;
    }
    yw+=w;
  }

  for (x=0;x<w;x++) {
    rsum=gsum=bsum=0;
    yp=-radius*w;
    for (i=-radius;i<=radius;i++) {
      yi=max(0, yp)+x;
      rsum+=r[yi];
      gsum+=g[yi];
      bsum+=b[yi];
      yp+=w;
    }
    yi=x;
    for (y=0;y<h;y++) {
      pix[yi]=0xff000000 | (dv[rsum]<<16) | (dv[gsum]<<8) | dv[bsum];
      if (x==0) {
        vmin[y]=min(y+radius+1, hm)*w;
        vmax[y]=max(y-radius, 0)*w;
      }
      p1=x+vmin[y];
      p2=x+vmax[y];

      rsum+=r[p1]-r[p2];
      gsum+=g[p1]-g[p2];
      bsum+=b[p1]-b[p2];

      yi+=w;
    }
  }
}

